"use strict";
/**
 * Objects - Form
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Form = void 0;
const uuid_1 = require("uuid");
const filters_1 = require("../../utils/filters/filters");
const isString_1 = require("../../utils/isString/isString");
const isObject_1 = require("../../utils/isObject/isObject");
const config_1 = require("../../config/config");
/**
 * Function - output form wrapper
 *
 * @param {import('./FormTypes').FormProps} props
 * @return {Promise<import('./FormTypes').FormReturn>}
 */
const Form = async (props) => {
    /* Fallback output */
    const fallback = {
        start: '',
        end: ''
    };
    /* Props must be object */
    if (!(0, isObject_1.isObjectStrict)(props)) {
        return fallback;
    }
    props = await (0, filters_1.applyFilters)('formProps', props, { renderType: 'form' });
    /* Filtered props must be object */
    if (!(0, isObject_1.isObjectStrict)(props)) {
        return fallback;
    }
    const { args } = props;
    const { id = '', action = 'sendForm', subject = '', toEmail = '', senderEmail = '', submitLabel = 'Send', successTitle = '', successText = '', errorTitle = '', errorText = '', errorSummary = '', errorResult = '', successResult = '', formClasses = '', formAttr = '', fieldsClasses = '', fieldsAttr = '', submitFieldClasses = '', submitClasses = '', submitAttr = '', submitLoader = '', honeypotFieldClasses = '', honeypotLabelClasses = '', honeypotClasses = '' } = (0, isObject_1.isObjectStrict)(args) ? args : {};
    /* Id required */
    if (!(0, isString_1.isStringStrict)(id)) {
        return fallback;
    }
    /* Add to form meta data */
    const meta = {};
    if ((0, isString_1.isStringStrict)(subject)) {
        meta.subject = subject;
    }
    if ((0, isString_1.isStringStrict)(toEmail)) {
        meta.toEmail = toEmail;
    }
    if ((0, isString_1.isStringStrict)(senderEmail)) {
        meta.senderEmail = senderEmail;
    }
    if (Object.keys(meta).length > 0) {
        config_1.config.formMeta[id] = meta;
    }
    /* Add to script data */
    const messages = {};
    if ((0, isString_1.isStringStrict)(successTitle)) {
        messages.successMessage = {
            primary: successTitle,
            secondary: successText
        };
    }
    if ((0, isString_1.isStringStrict)(errorTitle)) {
        messages.errorMessage = {
            primary: errorTitle,
            secondary: errorText
        };
    }
    if (Object.keys(messages).length > 0) {
        if (config_1.config.scriptMeta.forms === undefined) {
            config_1.config.scriptMeta.forms = {};
        }
        config_1.config.scriptMeta.forms[id] = messages;
    }
    config_1.config.scriptMeta.sendUrl = '/ajax/';
    /* Honeypot */
    const honeypotId = (0, uuid_1.v4)();
    const honeypotName = `${config_1.config.namespace}_asi`;
    const honeypot = `
    <div${(0, isString_1.isStringStrict)(honeypotFieldClasses) ? ` class="${honeypotFieldClasses}"` : ''} data-asi>
      <label${(0, isString_1.isStringStrict)(honeypotLabelClasses) ? ` class="${honeypotLabelClasses}"` : ''} for="${honeypotId}">Website</label>
      <input${(0, isString_1.isStringStrict)(honeypotClasses) ? ` class="${honeypotClasses}"` : ''} type="url" name="${honeypotName}" id="${honeypotId}" autocomplete="off">
    </div>
  `;
    /* Output */
    const start = `
    <form${(0, isString_1.isStringStrict)(formClasses) ? ` class="${formClasses}"` : ''} id="${id}" data-action="${action}"${(0, isString_1.isStringStrict)(formAttr) ? ` ${formAttr}` : ''} novalidate>
      <div${(0, isString_1.isStringStrict)(fieldsClasses) ? ` class="${fieldsClasses}"` : ''}${(0, isString_1.isStringStrict)(fieldsAttr) ? ` ${fieldsAttr}` : ''}>
        ${errorSummary}
  `;
    const end = `
        ${honeypot}
        ${errorResult}
        <div${(0, isString_1.isStringStrict)(submitFieldClasses) ? ` class="${submitFieldClasses}"` : ''} data-type="submit">
          <button${(0, isString_1.isStringStrict)(submitClasses) ? ` class="${submitClasses}"` : ''}${(0, isString_1.isStringStrict)(submitAttr) ? ` ${submitAttr}` : ''} type="submit">
            ${submitLoader}
            <span>${submitLabel}</span>
          </button>
        </div>
        ${successResult}
      </div>
    </form>
  `;
    return {
        start,
        end
    };
};
exports.Form = Form;
