"use strict";
/**
 * Serverless - Ajax
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Ajax = void 0;
const config_1 = require("../../config/config");
const actions_1 = require("../../utils/actions/actions");
const shortcodes_1 = require("../../utils/shortcodes/shortcodes");
const filters_1 = require("../../utils/filters/filters");
const isObject_1 = require("../../utils/isObject/isObject");
const isString_1 = require("../../utils/isString/isString");
const isNumber_1 = require("../../utils/isNumber/isNumber");
const SendForm_1 = require("../SendForm/SendForm");
/**
 * Class - custom exception to include status code
 *
 * @private
 */
class _CustomError extends Error {
    /**
     * Store message
     *
     * @type {string}
     */
    message;
    /**
     * Store status code
     *
     * @type {number}
     */
    httpStatusCode;
    /**
     * Set properties
     *
     * @param {import('./AjaxTypes').AjaxCustomErrorArgs} args
     */
    constructor(args) {
        if (!(0, isObject_1.isObjectStrict)(args)) {
            args = {};
        }
        const { message = '', code = 500 } = args;
        super(message);
        this.message = message;
        this.httpStatusCode = code;
    }
}
/**
 * Function - set env variables, normalize request body, check for required props and call actions
 *
 * @param {import('./AjaxTypes').AjaxArgs} args
 * @return {Promise<Response>} Response
 */
const Ajax = async ({ request, env, siteConfig }) => {
    try {
        /* Config */
        (0, config_1.setConfig)(siteConfig);
        await (0, config_1.setConfigFilter)(env);
        (0, filters_1.setFilters)(siteConfig.filters);
        (0, actions_1.setActions)(siteConfig.actions);
        (0, shortcodes_1.setShortcodes)(siteConfig.shortcodes);
        /* Get form data */
        const data = await request.json();
        /* Inputs required */
        if (data.inputs === undefined) {
            throw new Error('No inputs');
        }
        /* Honeypot check */
        const honeypotName = `${siteConfig.namespace}_asi`;
        if (data.inputs[honeypotName] !== undefined) {
            const honeypotValue = data.inputs[honeypotName].value;
            if ((0, isString_1.isStringStrict)(honeypotValue)) {
                const options = {
                    status: 200,
                    headers: {
                        'Content-Type': 'application/json'
                    }
                };
                return new Response(JSON.stringify({ success: 'Form successully sent' }), options);
            }
            data.inputs[honeypotName].exclude = true;
        }
        /* Id required */
        if (!(0, isString_1.isStringStrict)(data.id)) {
            throw new Error('No id');
        }
        /* Action required */
        const action = data.action !== undefined ? data.action : '';
        if (!(0, isString_1.isStringStrict)(action)) {
            throw new Error('No action');
        }
        /* Call functions by action */
        let res = null;
        if (action === 'sendForm') {
            res = await (0, SendForm_1.SendForm)({ ...data, env, request });
        }
        if (siteConfig.ajaxFunctions[action] !== undefined) {
            res = await siteConfig.ajaxFunctions[action]({ ...data, env, request });
        }
        const ajaxResFilterArgs = {
            action,
            ...data,
            env,
            request
        };
        res = await (0, filters_1.applyFilters)('ajaxRes', res, ajaxResFilterArgs);
        /* Result error */
        if (res === null) {
            throw new Error('No result');
        }
        if ((0, isString_1.isStringStrict)(res.error)) {
            throw new _CustomError(res.error);
        }
        /* Result success */
        const options = {
            status: 200,
            headers: {
                'Content-Type': 'application/json'
            }
        };
        let message = '';
        if (res.success !== undefined) {
            const { message: successMessage, headers } = res.success;
            if (successMessage !== undefined) {
                message = successMessage;
            }
            if (headers !== undefined) {
                options.headers = { ...options.headers, ...headers };
            }
        }
        return new Response(JSON.stringify({ success: message }), options);
    }
    catch (error) {
        console.error(siteConfig.console.red, '[SSF] Error with ajax function: ', error);
        let statusCode = 500;
        let message = '';
        if ((0, isObject_1.isObjectStrict)(error)) {
            const err = error;
            if ((0, isNumber_1.isNumber)(err.httpStatusCode)) {
                statusCode = err.httpStatusCode;
            }
            if ((0, isString_1.isStringStrict)(err.message)) {
                message = err.message;
            }
        }
        const options = {
            status: statusCode,
            headers: {
                'Content-Type': 'application/json'
            }
        };
        return new Response(JSON.stringify({ error: message }), options);
    }
};
exports.Ajax = Ajax;
