"use strict";
/**
 * Utils - Actions
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.setActions = exports.resetActions = exports.doActions = exports.removeAction = exports.addAction = void 0;
const isString_1 = require("../isString/isString");
const isArray_1 = require("../isArray/isArray");
const isObject_1 = require("../isObject/isObject");
const isFunction_1 = require("../isFunction/isFunction");
/**
 * Store action callbacks by name
 *
 * @type {import('./actionsTypes').ActionsFunctions}
 */
let actions = {
    renderStart: [],
    renderEnd: [],
    renderItemStart: [],
    renderItemEnd: []
};
/**
 * Function - add action to action object
 *
 * @param {string} name
 * @param {function} action
 * @return {boolean}
 */
const addAction = (name, action) => {
    if (!(0, isString_1.isStringStrict)(name) || !(0, isFunction_1.isFunction)(action)) {
        return false;
    }
    if (actions[name] === undefined) {
        actions[name] = [];
    }
    actions[name].push(action);
    return true;
};
exports.addAction = addAction;
/**
 * Function - remove action from actions object
 *
 * @param {string} name
 * @param {function} action
 * @return {boolean}
 */
const removeAction = (name, action) => {
    if (!(0, isString_1.isStringStrict)(name) || !(0, isFunction_1.isFunction)(action)) {
        return false;
    }
    const callbacks = actions[name];
    if ((0, isArray_1.isArrayStrict)(callbacks)) {
        const index = callbacks.indexOf(action);
        if (index > -1) {
            actions[name].splice(index, 1);
            return true;
        }
    }
    return false;
};
exports.removeAction = removeAction;
/**
 * Function - run callback functions from actions object
 *
 * @param {string} name
 * @param {*} [args]
 * @return {Promise<void>}
 */
const doActions = async (name, args) => {
    const callbacks = actions[name];
    if ((0, isArray_1.isArrayStrict)(callbacks)) {
        for (let i = 0; i < callbacks.length; i += 1) {
            const callback = callbacks[i];
            if ((0, isFunction_1.isFunction)(callback)) {
                await callback(args);
            }
        }
    }
};
exports.doActions = doActions;
/**
 * Function - empty actions object
 *
 * @return {void}
 */
const resetActions = () => {
    actions = {
        renderStart: [],
        renderEnd: [],
        renderItemStart: [],
        renderItemEnd: []
    };
};
exports.resetActions = resetActions;
/**
 * Function - fill actions object
 *
 * @param {import('./actionsTypes').Actions} args
 * @return {boolean}
 */
const setActions = (args) => {
    if (!(0, isObject_1.isObjectStrict)(args)) {
        return false;
    }
    if (Object.keys(args).length === 0) {
        return false;
    }
    resetActions();
    Object.keys(args).forEach((a) => {
        const arg = args[a];
        if (arg === undefined) {
            return;
        }
        addAction(a, arg);
    });
    return true;
};
exports.setActions = setActions;
