"use strict";
/**
 * Utils - Get Contentful Data
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getContentfulData = void 0;
const contentful_resolve_response_1 = require("contentful-resolve-response");
const normalizeContentfulData_1 = require("../normalizeContentfulData/normalizeContentfulData");
const filters_1 = require("../filters/filters");
const isObject_1 = require("../isObject/isObject");
const isString_1 = require("../isString/isString");
const config_1 = require("../../config/config");
/**
 * Function - fetch data from contentful cms or cache
 *
 * @param {string} key
 * @param {import('./getContentfulDataTypes').ContentfulDataParams} params
 * @return {Promise<import('./getContentfulDataTypes').ContentfulDataReturn>}
 */
const getContentfulData = async (key = '', params = {}) => {
    try {
        /* Key required for cache */
        if (!(0, isString_1.isStringStrict)(key)) {
            throw new Error('No key');
        }
        /* Check cache */
        if (config_1.config.env.cache) {
            let cacheData = {};
            const cacheDataFilterArgs = {
                key,
                type: 'get'
            };
            cacheData = await (0, filters_1.applyFilters)('cacheData', cacheData, cacheDataFilterArgs);
            if ((0, isObject_1.isObject)(cacheData)) {
                return structuredClone(cacheData);
            }
        }
        /* Credentials */
        const { space, previewAccessToken, deliveryAccessToken, previewHost, deliveryHost } = config_1.config.cms;
        let accessToken = previewAccessToken;
        let host = previewHost;
        if (config_1.config.env.prod) {
            accessToken = deliveryAccessToken;
            host = deliveryHost;
        }
        if (!(0, isString_1.isStringStrict)(space) || !(0, isString_1.isStringStrict)(accessToken) || !(0, isString_1.isStringStrict)(host)) {
            throw new Error('No credentials');
        }
        /* Params */
        let url = `https://${host}/spaces/${space}/environments/master/entries?access_token=${accessToken}`;
        Object.keys(params).forEach(p => {
            url += `&${p}=${params[p].toString()}`;
        });
        /* Check and transform data */
        const resp = await fetch(url);
        const data = await resp.json();
        if (!(0, isObject_1.isObjectStrict)(data)) {
            throw new Error('No data');
        }
        if (data.items === undefined) {
            throw new Error('No items');
        }
        data.items = (0, contentful_resolve_response_1.default)(data);
        const newItems = (0, normalizeContentfulData_1.normalizeContentfulData)(data.items !== undefined ? data.items : []);
        const newData = {
            items: newItems
        };
        /* Store in cache */
        if (config_1.config.env.cache) {
            const cacheDataFilterArgs = {
                key,
                type: 'set',
                data
            };
            await (0, filters_1.applyFilters)('cacheData', newData, cacheDataFilterArgs);
        }
        /* Output */
        return newData;
    }
    catch (error) {
        console.error(config_1.config.console.red, '[SSF] Error fetching Contentful data: ', error);
        /* Store in cache (avoid extra calls when no result) */
        if (config_1.config.env.cache) {
            const cacheDataFilterArgs = {
                key,
                type: 'set',
                data: {}
            };
            await (0, filters_1.applyFilters)('cacheData', {}, cacheDataFilterArgs);
        }
        /* Output */
        return {};
    }
};
exports.getContentfulData = getContentfulData;
