"use strict";
/**
 * Utils - Get File Data
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFileData = void 0;
const promises_1 = require("node:fs/promises");
const node_path_1 = require("node:path");
const filters_1 = require("../../utils/filters/filters");
const isObject_1 = require("../../utils/isObject/isObject");
const isString_1 = require("../../utils/isString/isString");
const getJson_1 = require("../../utils/getJson/getJson");
const config_1 = require("../../config/config");
/**
 * Function - get data from file or cache
 *
 * @param {string} key
 * @param {import('./getFileDataTypes').FileDataParams} params
 * @return {Promise<import('./getFileDataTypes').FileDataReturn>}
 */
const getFileData = async (key = '', params = {}) => {
    try {
        /* Key required for cache */
        if (!(0, isString_1.isStringStrict)(key)) {
            throw new Error('No key');
        }
        /* Check cache */
        if (config_1.config.env.cache) {
            let cacheData = {};
            const cacheDataFilterArgs = {
                key,
                type: 'get'
            };
            cacheData = await (0, filters_1.applyFilters)('cacheData', cacheData, cacheDataFilterArgs);
            if ((0, isObject_1.isObject)(cacheData) && Object.keys(cacheData).length > 0) {
                return structuredClone(cacheData);
            }
        }
        /* Params */
        const { all = true, id = '' } = params;
        /* Store data */
        const data = {};
        /* Single file */
        if ((0, isString_1.isStringStrict)(id) && !all) {
            const file = await (0, promises_1.readFile)((0, node_path_1.resolve)(config_1.config.static.dir, `${id}.json`), { encoding: 'utf8' });
            const fileJson = (0, getJson_1.getJson)(file);
            if (fileJson !== undefined) {
                data[id] = fileJson;
            }
        }
        /* All files */
        if (!(0, isString_1.isStringStrict)(id) && all) {
            const files = await (0, promises_1.readdir)((0, node_path_1.resolve)(config_1.config.static.dir));
            for (let i = 0; i < files.length; i += 1) {
                const file = files[i];
                const fileExt = (0, node_path_1.extname)(file);
                const fileName = (0, node_path_1.basename)(file, fileExt);
                if (fileExt === '.json') {
                    const fileContents = await (0, promises_1.readFile)((0, node_path_1.resolve)(config_1.config.static.dir, file), { encoding: 'utf8' });
                    const fileJson = (0, getJson_1.getJson)(fileContents);
                    if (fileJson !== undefined) {
                        data[fileName] = fileJson;
                    }
                }
            }
        }
        /* Data not empty check */
        if (Object.keys(data).length === 0) {
            throw new Error('No file data');
        }
        /* Store in cache */
        if (config_1.config.env.cache) {
            const cacheDataFilterArgs = {
                key,
                type: 'set',
                data
            };
            await (0, filters_1.applyFilters)('cacheData', data, cacheDataFilterArgs);
        }
        /* Output */
        return data;
    }
    catch (error) {
        console.error(config_1.config.console.red, '[SSF] Error getting file data: ', error);
        return {};
    }
};
exports.getFileData = getFileData;
