"use strict";
/**
 * Utils - Process Images
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.processImages = void 0;
const sharp_1 = require("sharp");
const promises_1 = require("node:fs/promises");
const node_path_1 = require("node:path");
const config_1 = require("../../config/config");
const getAllFilePaths_1 = require("../getAllFilePaths/getAllFilePaths");
const isString_1 = require("../isString/isString");
/**
 * Function - get and save image data and output multiple sizes
 *
 * @return {Promise<PromiseSettledResult<sharp.OutputInfo>[]>}
 */
const processImages = async () => {
    const store = {};
    try {
        const inputDir = config_1.config.static.image.inputDir;
        const outputDir = config_1.config.static.image.outputDir;
        const dataFile = config_1.config.static.image.dataFile;
        if (!(0, isString_1.isStringStrict)(inputDir) || !(0, isString_1.isStringStrict)(outputDir)) {
            throw new Error('No input or output directories');
        }
        await (0, promises_1.mkdir)((0, node_path_1.resolve)(outputDir), { recursive: true });
        const sharpImages = [];
        for await (const path of (0, getAllFilePaths_1.getAllFilePaths)(inputDir)) {
            if (!path.includes('.DS_Store')) {
                const ext = (0, node_path_1.extname)(path);
                const baseName = (0, node_path_1.basename)(path);
                const base = baseName.split(ext)[0];
                /* Nested files */
                let folders = (0, node_path_1.dirname)(path).split(`${inputDir}/`)[1];
                if (folders !== undefined && folders !== baseName) {
                    await (0, promises_1.mkdir)((0, node_path_1.resolve)(outputDir, folders), { recursive: true });
                }
                else {
                    folders = '';
                }
                /* Store meta data */
                const metadata = await (0, sharp_1.default)(path).metadata();
                const id = `${folders !== '' ? `${folders}/` : ''}${base}`;
                const format = ext.split('.')[1];
                const { width = 0, height = 0, size: fileSize = 0, format: fileFormat = 'jpeg' } = metadata;
                store[id] = {
                    path: id,
                    name: baseName,
                    size: fileSize,
                    type: `image/${fileFormat}`,
                    format,
                    width,
                    height
                };
                /* Sizes */
                let sizes = [...config_1.config.image.sizes];
                sizes.push(width);
                sizes = sizes.filter(s => s <= width);
                sizes.forEach((size) => {
                    sharpImages.push({
                        size,
                        ext: format,
                        path: (0, node_path_1.resolve)(path),
                        newPath: (0, node_path_1.resolve)(outputDir, folders, `${base}${size !== width ? `@${size}` : ''}`)
                    });
                });
            }
        }
        if (sharpImages.length === 0) {
            throw new Error('No images to process');
        }
        if ((0, isString_1.isStringStrict)(dataFile)) {
            await (0, promises_1.writeFile)((0, node_path_1.resolve)(dataFile), JSON.stringify(store));
        }
        return await Promise.allSettled(sharpImages.map(async (c) => {
            const { size, path, newPath, ext } = c;
            await (0, sharp_1.default)(path)
                .resize(size)
                .toFile(`${newPath}.${ext}`);
            const create = await (0, sharp_1.default)(path)
                .webp({ quality: config_1.config.image.quality })
                .resize(size)
                .toFile(`${newPath}.webp`);
            return create;
        }));
    }
    catch (error) {
        console.error(config_1.config.console.red, '[SSF] Error processing images: ', error);
    }
    return [];
};
exports.processImages = processImages;
