"use strict";
/**
 * Objects - Field
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Field = void 0;
const uuid_1 = require("uuid");
const filters_1 = require("../../utils/filters/filters");
const isString_1 = require("../../utils/isString/isString");
const isObject_1 = require("../../utils/isObject/isObject");
const isArray_1 = require("../../utils/isArray/isArray");
/**
 * Function - output checkbox and radio inputs from options
 *
 * @private
 * @param {import('./FieldTypes').FieldCheckboxRadioArgs} args
 * @return {string[]}
 */
const _getCheckboxRadioOpts = (args = {}) => {
    const { opts = [], name = '', classes = '', attr = '', type = 'checkbox', icon = '', labelClass = '' } = args;
    /* Opts and name required */
    if (opts.length === 0 || name === '') {
        return '';
    }
    /* Output */
    return opts.map((opt) => {
        const { text = '', value = '', selected = false } = opt;
        const id = (0, uuid_1.v4)();
        return `
      <div data-option-field data-type="${type}">
        <input type="${type}" name="${name}" id="${id}" class="${classes}" value="${value}"${attr}${selected ? ' checked' : ''}>
        <label for="${id}" data-option>
          <span${labelClass} data-label>
            <span data-label-text>${text}</span>
          </span>
          <span data-control data-type="${type}">${icon}</span>
        </label>
      </div>
    `;
    }).join('');
};
/**
 * Function - output form field
 *
 * @param {import('./FieldTypes').FieldProps} props
 * @return {Promise<string>} HTML - div
 */
const Field = async (props) => {
    /* Props must be object */
    if (!(0, isObject_1.isObjectStrict)(props)) {
        return '';
    }
    props = await (0, filters_1.applyFilters)('fieldProps', props, { renderType: 'field' });
    /* Filtered props must be object */
    if (!(0, isObject_1.isObjectStrict)(props)) {
        return '';
    }
    let { args } = props;
    args = (0, isObject_1.isObjectStrict)(args) ? args : {};
    const { type = 'text', name = '', label = '', value = '', required = false, width = '', widthSmall = '', widthMedium = '', widthLarge = '', autoCompleteToken = '', placeholder = '', options = [], rows = 5, emptyErrorMessage = '', invalidErrorMessage = '', fieldsetClasses = '', fieldClasses = '', labelClasses = '', classes = '', visuallyHiddenClass = '', radioIcon = '', checkboxIcon = '', selectIcon = '' } = args;
    let { fieldset = false } = args;
    /* Name and label required */
    if (!(0, isString_1.isStringStrict)(name) || !(0, isString_1.isStringStrict)(label)) {
        return '';
    }
    /* Id */
    const id = (0, uuid_1.v4)();
    /* Field classes */
    const fieldClassesArr = [];
    if ((0, isString_1.isStringStrict)(fieldClasses)) {
        fieldClassesArr.push(fieldClasses);
    }
    /* Width */
    if ((0, isString_1.isStringStrict)(width)) {
        fieldClassesArr.push(width);
    }
    if ((0, isString_1.isStringStrict)(widthSmall) && widthSmall !== width) {
        fieldClassesArr.push(widthSmall);
    }
    if ((0, isString_1.isStringStrict)(widthMedium) && widthMedium !== widthSmall) {
        fieldClassesArr.push(widthMedium);
    }
    if ((0, isString_1.isStringStrict)(widthLarge) && widthLarge !== widthMedium) {
        fieldClassesArr.push(widthLarge);
    }
    /* Classes */
    const classesArr = [];
    if ((0, isString_1.isStringStrict)(classes)) {
        classesArr.push(classes);
    }
    /* Checkbox or radio */
    const checkboxRadio = type === 'checkbox' || type === 'radio';
    /* Icon */
    const icon = type === 'radio' ? radioIcon : checkboxIcon;
    /* Options */
    const opts = [];
    if ((0, isArray_1.isArrayStrict)(options)) {
        options.forEach((option) => {
            const data = option.split(' : ');
            if (data.length >= 2) {
                opts.push({
                    text: data[0],
                    value: data[1],
                    selected: data.length === 3
                });
            }
        });
    }
    /* Check if fieldset */
    let checkboxRadioOpts = false;
    if (opts.length > 0 && checkboxRadio) {
        checkboxRadioOpts = true;
        fieldset = true;
    }
    /* Attributes */
    const attr = [];
    if (required) {
        attr.push(fieldset ? 'data-aria-required="true"' : 'aria-required="true"');
    }
    if ((0, isString_1.isStringStrict)(value) && !checkboxRadioOpts) {
        attr.push(`value="${value}"`);
    }
    if ((0, isString_1.isStringStrict)(placeholder)) {
        attr.push(`placeholder="${placeholder}"`);
    }
    if ((0, isString_1.isStringStrict)(autoCompleteToken)) {
        attr.push(`autocomplete="${autoCompleteToken}"`);
    }
    if ((0, isString_1.isStringStrict)(emptyErrorMessage)) {
        attr.push(`data-empty-message="${emptyErrorMessage}"`);
    }
    if ((0, isString_1.isStringStrict)(invalidErrorMessage)) {
        attr.push(`data-invalid-message="${invalidErrorMessage}"`);
    }
    if (classesArr.length > 0) {
        attr.push(`class="${classesArr.join(' ')}"`);
    }
    if (rows > 0 && type === 'textarea') {
        attr.push(`rows="${rows}"`);
    }
    let attrs = '';
    if (attr.length > 0) {
        attrs = ` ${attr.join(' ')}`;
    }
    /* Label */
    let labelBefore = '';
    let labelAfter = '';
    const labelRequired = required ? ' data-required' : '';
    const labelRequiredIcon = required ? '<span data-required-icon aria-hidden="true"></span>' : '';
    const labelClass = (0, isString_1.isStringStrict)(labelClasses) ? ` class="${labelClasses}"` : '';
    if (checkboxRadio) {
        if (fieldset) {
            labelBefore = `
        <legend${labelRequired}>
          <span data-legend-text>${label}${required ? `<span${(0, isString_1.isStringStrict)(visuallyHiddenClass) ? ` class="${visuallyHiddenClass}"` : ''}> required</span>` : ''}
            ${labelRequiredIcon}
          </span>
        </legend>
      `;
        }
        else {
            labelAfter = `
        <label for="${id}" data-option>
          <span${labelClass} data-label${labelRequired}>
            <span data-label-text>
              ${label}
              ${labelRequiredIcon}
            </span>
          </span>
          <span data-control data-type="${type}">${icon}</span>
        </label>
      `;
        }
    }
    else {
        labelBefore = `
      <label for="${id}"${labelClass} data-label${labelRequired}>
        <span data-label-text>
          ${label}
          ${labelRequiredIcon}
        </span>
      </label>
    `;
    }
    /* Input */
    let input = '';
    switch (type) {
        case 'text':
        case 'email':
        case 'checkbox':
        case 'radio':
        case 'number':
        case 'password':
        case 'tel': {
            input = `<input type="${type}" name="${name}" id="${id}"${attrs}>`;
            if (checkboxRadioOpts) {
                input = _getCheckboxRadioOpts({
                    opts,
                    name,
                    classes,
                    attr: attrs,
                    type,
                    icon,
                    labelClass
                });
            }
            break;
        }
        case 'textarea': {
            input = `<textarea name="${name}" id="${id}"${attrs}></textarea>`;
            break;
        }
        case 'select': {
            if (opts.length > 0) {
                const optsOutput = opts.map((opt) => {
                    const { text, value, selected = false } = opt;
                    return `<option value="${value}"${selected ? ' selected' : ''}>${text}</option>`;
                }).join('');
                input = `
          <div data-type="select">
            <select name="${name}" id="${id}"${attrs}>${optsOutput}</select>
            <div data-select-icon>${selectIcon}</div>
          </div>
        `;
            }
            break;
        }
    }
    if (input === '') {
        return '';
    }
    /* Output */
    return `
    <div class="${fieldClassesArr.join(' ')}" data-type="${type}">
      ${fieldset ? `<fieldset${(0, isString_1.isStringStrict)(fieldsetClasses) ? ` class="${fieldsetClasses}"` : ''}>` : ''}
      ${labelBefore}
      ${input}
      ${labelAfter}
      ${fieldset ? '</fieldset>' : ''}
    </div>
  `;
};
exports.Field = Field;
