"use strict";
/**
 * Render
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRenderFunctions = exports.renderContent = exports.renderItem = exports.render = void 0;
const normalizeContentType_1 = require("../utils/normalizeContentType/normalizeContentType");
const actions_1 = require("../utils/actions/actions");
const filters_1 = require("../utils/filters/filters");
const getSlug_1 = require("../utils/getSlug/getSlug");
const getPermalink_1 = require("../utils/getPermalink/getPermalink");
const getPath_1 = require("../utils/getPath/getPath");
const getJson_1 = require("../utils/getJson/getJson");
const isString_1 = require("../utils/isString/isString");
const isArray_1 = require("../utils/isArray/isArray");
const isObject_1 = require("../utils/isObject/isObject");
const isNumber_1 = require("../utils/isNumber/isNumber");
const isFunction_1 = require("../utils/isFunction/isFunction");
const shortcodes_1 = require("../utils/shortcodes/shortcodes");
const tag_1 = require("../utils/tag/tag");
const config_1 = require("../config/config");
const Container_1 = require("../layouts/Container/Container");
const Column_1 = require("../layouts/Column/Column");
const Form_1 = require("../objects/Form/Form");
const Field_1 = require("../objects/Field/Field");
const RichText_1 = require("../text/RichText/RichText");
/**
 * Store slug data
 *
 * @private
 * @type {import('./RenderTypes').RenderSlugs}
 */
const _slugs = {};
/**
 * Function - normalize meta properties into one object
 *
 * @private
 * @param {import('./RenderTypes').RenderMetaArgs} args
 * @return {import('./RenderTypes').RenderMetaReturn}
 */
const _getMeta = (args) => {
    const meta = {
        title: '',
        description: '',
        url: '',
        image: '',
        canonical: '',
        prev: '',
        next: '',
        noIndex: false,
        isIndex: false
    };
    if (!(0, isObject_1.isObjectStrict)(args)) {
        return meta;
    }
    if ((0, isObject_1.isObjectStrict)(args.meta)) {
        return Object.assign(meta, args.meta);
    }
    if ((0, isString_1.isStringStrict)(args.metaTitle)) {
        meta.title = args.metaTitle;
    }
    if ((0, isString_1.isStringStrict)(args.metaDescription)) {
        meta.description = args.metaDescription;
    }
    if ((0, isString_1.isStringStrict)(args?.metaImage?.url)) {
        meta.image = args.metaImage.url;
    }
    return meta;
};
/**
 * Function - add pagination data to meta object
 *
 * @private
 * @param {import('../components/Pagination/PaginationTypes').PaginationData} args
 * @return {void}
 */
const _setPaginationMeta = (args, slugArgs, meta) => {
    const { current = 0, currentFilters, prevFilters, nextFilters } = args;
    slugArgs.page = current > 1 ? current : 0;
    const s = (0, getSlug_1.getSlug)(slugArgs);
    if ((0, isObject_1.isObject)(s)) {
        meta.canonical = `${(0, getPermalink_1.getPermalink)(s.slug, current === 1)}${(0, isString_1.isString)(currentFilters) ? currentFilters : ''}`;
    }
    if ((0, isString_1.isStringStrict)(args.title)) {
        meta.paginationTitle = args.title;
    }
    if ((0, isNumber_1.isNumber)(args.prev)) {
        slugArgs.page = args.prev > 1 ? args.prev : 0;
        const p = (0, getSlug_1.getSlug)(slugArgs);
        if ((0, isObject_1.isObject)(p)) {
            meta.prev = `${(0, getPermalink_1.getPermalink)(p.slug, args.prev === 1)}${(0, isString_1.isString)(prevFilters) ? prevFilters : ''}`;
        }
    }
    if ((0, isNumber_1.isNumber)(args.next) && args.next > 1) {
        slugArgs.page = args.next;
        const n = (0, getSlug_1.getSlug)(slugArgs);
        if ((0, isObject_1.isObject)(n)) {
            meta.next = `${(0, getPermalink_1.getPermalink)(n.slug, false)}${(0, isString_1.isString)(nextFilters) ? nextFilters : ''}`;
        }
    }
};
/**
 * Function - get content and templates in expected format to map
 *
 * @private
 * @param {import('./RenderTypes').RenderItem[]} content
 * @param {import('./RenderTypes').RenderItem[]} [_templates]
 * @return {import('./RenderTypes').RenderContentTemplate}
 */
const _getContentTemplate = (content, _templates = []) => {
    /* Content must be array */
    if (!(0, isArray_1.isArrayStrict)(content)) {
        return {
            content: [],
            templates: _templates
        };
    }
    /* One level loop */
    const _content = content.map((c) => {
        /* Check if template */
        if ((0, tag_1.tagExists)(c, 'template')) {
            _templates.push(structuredClone(c));
            /* Replace template with template break */
            return {
                metadata: {
                    tags: [
                        {
                            id: 'templateBreak',
                            name: ''
                        }
                    ]
                }
            };
        }
        /* Output */
        return c;
    });
    /* Output */
    return {
        content: _content,
        templates: _templates
    };
};
/**
 * Function - map out content slots to templates for contentTemplate
 *
 * @private
 * @param {import('./RenderTypes').RenderItem[]} templates
 * @param {import('./RenderTypes').RenderItem[]} [content]
 * @return {import('./RenderTypes').RenderItem[]}
 */
const _mapContentTemplate = (templates, content = []) => {
    /* Templates must be arrays */
    if (!(0, isArray_1.isArrayStrict)(templates)) {
        return templates;
    }
    /* Recurse templates */
    const lastTemplateIndex = templates.length - 1;
    templates.forEach((t, i) => {
        /* Remove template break */
        if ((0, tag_1.tagExists)(content[0], 'templateBreak') && content.length >= 1) {
            content.shift();
        }
        /* Check if optional - remove if no content */
        const isOptional = (0, tag_1.tagExists)(t, 'templateOptional');
        if (isOptional && i === lastTemplateIndex && content.length === 0) {
            templates.pop();
            return;
        }
        /* Check if slot */
        const isSlot = (0, tag_1.tagExists)(t, 'templateSlot');
        /* Check for repeatable template item */
        const children = t.content;
        if ((0, isArray_1.isArrayStrict)(children) && !isSlot) {
            let repeat;
            const repeatIndex = children.findIndex((c) => {
                const isRepeat = (0, tag_1.tagExists)(c, 'templateRepeat');
                if (isRepeat) {
                    repeat = c;
                }
                return isRepeat;
            });
            if (repeatIndex !== -1 && repeat !== undefined) {
                let breakIndex = content.findIndex((c) => {
                    return (0, tag_1.tagExists)(c, 'templateBreak');
                });
                breakIndex = breakIndex === -1 ? content.length : breakIndex;
                let insertIndex = repeatIndex;
                for (let j = insertIndex; j < breakIndex - 1; j += 1) {
                    children.splice(insertIndex, 0, structuredClone(repeat));
                    insertIndex = j;
                }
            }
        }
        /* Replace slot with content */
        if (isSlot && content.length >= 1) {
            const fill = content.shift();
            if (fill !== undefined) {
                templates[i] = fill;
            }
            return;
        }
        /* Recurse children */
        if ((0, isArray_1.isArray)(children)) {
            templates[i].content = _mapContentTemplate(children, content);
        }
    });
    /* Output */
    return templates;
};
/**
 * Function - get config and formation render functions
 *
 * @return {import('../global/globalTypes').GenericFunctions}
 */
const getRenderFunctions = () => {
    const renderFunctions = { ...config_1.config.renderFunctions };
    if (renderFunctions.container === undefined) {
        renderFunctions.container = Container_1.Container;
    }
    if (renderFunctions.column === undefined) {
        renderFunctions.column = Column_1.Column;
    }
    if (renderFunctions.form === undefined) {
        renderFunctions.form = Form_1.Form;
    }
    if (renderFunctions.field === undefined) {
        renderFunctions.field = Field_1.Field;
    }
    if (renderFunctions.richText === undefined) {
        renderFunctions.richText = RichText_1.RichText;
    }
    return renderFunctions;
};
exports.getRenderFunctions = getRenderFunctions;
/**
 * Function - recurse and output nested content
 *
 * @param {import('./RenderTypes').RenderContentArgs} args
 * @param {import('../global/globalTypes').HtmlString} [_output]
 * @return {Promise<string>}
 */
const renderContent = async (args, _output = { html: '' }) => {
    if (!(0, isObject_1.isObjectStrict)(args)) {
        return _output.html;
    }
    const { content = [], serverlessData, pageData, pageContains = [], pageHeadings = [], navigations = {}, renderFunctions = {} } = args;
    let { parents = [], headingsIndex = 0, depth = 0 } = args;
    /* Content must be array */
    if (!(0, isArray_1.isArrayStrict)(content)) {
        return _output.html;
    }
    /* Loop */
    for (let i = 0; i < content.length; i += 1) {
        const item = content[i];
        if (!(0, isObject_1.isObjectStrict)(item)) {
            continue;
        }
        /* Render props */
        const props = structuredClone(item);
        const renderType = (0, isString_1.isString)(props.renderType) ? props.renderType : '';
        const contentType = (0, isString_1.isString)(props.contentType) ? props.contentType : '';
        /* Check for nested content */
        let children = props.content;
        /* Map out content to template */
        if (contentType === 'contentTemplate') {
            const contentTemplate = _getContentTemplate((0, isArray_1.isArray)(props.content) ? props.content : []);
            const templates = _mapContentTemplate(contentTemplate.templates, contentTemplate.content);
            children = templates;
        }
        /* Children array check */
        let childrenArr;
        if ((0, isArray_1.isArrayStrict)(children) && renderType !== 'richText') {
            childrenArr = children;
        }
        /* Render output */
        let renderObj = {
            start: '',
            end: ''
        };
        let filterType = '';
        let filterArgs = {};
        const renderFunction = renderFunctions[renderType];
        if ((0, isFunction_1.isFunction)(renderFunction)) {
            const renderArgs = {
                args: props,
                parents,
                pageData,
                pageContains,
                navigations,
                serverlessData
            };
            if (childrenArr !== undefined) {
                renderArgs.children = childrenArr;
            }
            if (renderType === 'richText') {
                renderArgs.headings = pageHeadings[headingsIndex];
            }
            const renderOutput = await renderFunction(renderArgs);
            if ((0, isString_1.isString)(renderOutput)) {
                renderObj.start = renderOutput;
            }
            else {
                renderObj = renderOutput;
            }
            pageContains.push(renderType);
            filterType = renderType;
            filterArgs = {
                ...props,
                content: undefined
            };
        }
        let start = renderObj.start;
        let end = renderObj.end;
        /* Filter start and end output */
        const renderContentFilterArgs = {
            renderType: filterType,
            args: filterArgs
        };
        if (start !== '' && end === '') {
            start = await (0, filters_1.applyFilters)('renderContent', start, renderContentFilterArgs);
        }
        else {
            start = await (0, filters_1.applyFilters)('renderContentStart', start, renderContentFilterArgs);
            end = await (0, filters_1.applyFilters)('renderContentEnd', end, renderContentFilterArgs);
        }
        /* Add to output object */
        _output.html += start;
        /* Recurse through children */
        if (childrenArr !== undefined) {
            const parentsCopy = [...parents];
            parentsCopy.unshift({
                renderType,
                args: {
                    ...props,
                    content: undefined,
                    parents: undefined
                }
            });
            await renderContent({
                content: childrenArr,
                serverlessData,
                parents: parentsCopy,
                pageData,
                pageContains,
                pageHeadings,
                navigations,
                renderFunctions,
                headingsIndex,
                depth: depth += 1
            }, _output);
        }
        _output.html += end;
        /* Additional rich text areas */
        if (renderType === 'content' && depth === 0) {
            headingsIndex = pageHeadings.push([]);
        }
    }
    /* Output */
    return _output.html;
};
exports.renderContent = renderContent;
/**
 * Function - output single post or page
 *
 * @param {import('./RenderTypes').RenderItemArgs} args
 * @return {import('./RenderTypes').RenderItemReturn}
 */
const renderItem = async (args) => {
    if (!(0, isObject_1.isObjectStrict)(args)) {
        return {};
    }
    const { item, contentType = 'page', serverlessData, renderFunctions } = args;
    /* Item must be object */
    if (!(0, isObject_1.isObjectStrict)(item)) {
        return {};
    }
    /* Taxonomy not page */
    if (contentType === 'taxonomy' && item.isPage !== true) {
        return {};
    }
    /* Item id required */
    const id = item.id;
    if (!(0, isString_1.isStringStrict)(id)) {
        return {};
    }
    /* Slug required */
    if (!(0, isString_1.isStringStrict)(item.slug)) {
        return {};
    }
    /* Serverless render check */
    let serverlessRender = false;
    /* Store components contained in page  */
    const pageContains = [];
    /* Store rich text headings in page */
    const pageHeadings = [[]];
    /* Start action */
    const renderItemStartArgs = {
        id,
        pageData: item,
        contentType,
        pageContains,
        pageHeadings,
        serverlessData
    };
    await (0, actions_1.doActions)('renderItemStart', renderItemStartArgs);
    /* Reset script and style files */
    config_1.config.scripts.item = {};
    config_1.config.styles.item = {};
    /* Meta */
    const title = item.title;
    const meta = _getMeta(item);
    if (!(0, isString_1.isStringStrict)(meta.title)) {
        meta.title = title;
    }
    /* Term taxonomy */
    const taxonomy = item.taxonomy;
    /* Permalink */
    const slugArgs = {
        id,
        contentType,
        slug: item.slug,
        returnParents: true,
        pageData: item,
        page: 0
    };
    const s = (0, getSlug_1.getSlug)(slugArgs);
    let slug = '';
    let permalink = '';
    let parents = [];
    if (!(0, isString_1.isString)(s)) {
        slug = s.slug;
        parents = s.parents;
        permalink = (0, getPermalink_1.getPermalink)(slug);
        item.basePermalink = (0, getPermalink_1.getPermalink)(slug);
    }
    meta.url = permalink;
    meta.canonical = permalink;
    /* Add to data by slugs store */
    let formattedSlug = slug !== 'index' && slug !== '' ? `/${slug}/` : '/';
    if (slug.includes('.html')) {
        formattedSlug = slug;
    }
    _slugs[formattedSlug] = {
        contentType,
        id
    };
    /* Check if index */
    const index = item.slug === 'index';
    meta.isIndex = index;
    /* Navigations */
    let navigations = {};
    if ((0, isFunction_1.isFunction)(renderFunctions.navigations)) {
        let currentType = contentType;
        if ((0, isObject_1.isObjectStrict)(taxonomy)) {
            currentType = (0, isString_1.isStringStrict)(taxonomy.contentType) ? taxonomy.contentType : contentType;
        }
        navigations = renderFunctions.navigations({
            navigations: config_1.config.navigation,
            items: config_1.config.navigationItem,
            currentLink: permalink,
            currentType,
            title,
            parents
        });
    }
    /* Serverless data */
    let itemServerlessData;
    if ((0, isObject_1.isObjectStrict)(serverlessData)) {
        const serverlessPath = serverlessData.path !== undefined ? serverlessData.path : '';
        if (serverlessPath === formattedSlug && serverlessData.query !== undefined) {
            itemServerlessData = serverlessData;
        }
        else { // Avoid re-rendering non dynamic pages
            return {
                serverlessRender: false
            };
        }
    }
    /* Page data (props) for layout and actions */
    const pageData = { ...item };
    pageData.id = id;
    pageData.parents = parents;
    pageData.content = undefined;
    /* Content loop */
    let contentOutput = '';
    const contentData = item.content;
    if ((0, isArray_1.isArrayStrict)(contentData)) {
        contentOutput = await renderContent({
            content: contentData,
            serverlessData: itemServerlessData,
            parents: [],
            pageData,
            pageContains,
            pageHeadings,
            navigations,
            renderFunctions
        });
    }
    contentOutput = await (0, shortcodes_1.doShortcodes)(contentOutput);
    /* Pagination variables for meta object */
    if ((0, isObject_1.isObjectStrict)(pageData.pagination)) {
        _setPaginationMeta(pageData.pagination, slugArgs, meta);
        serverlessRender = true;
    }
    /* Output */
    let layoutOutput = '';
    if ((0, isFunction_1.isFunction)(renderFunctions.layout)) {
        const layoutArgs = {
            id,
            meta,
            navigations,
            contentType,
            content: contentOutput,
            slug: formattedSlug,
            pageContains,
            pageHeadings,
            pageData,
            serverlessData
        };
        layoutOutput = await renderFunctions.layout(layoutArgs);
    }
    const renderItemFilterArgs = {
        id,
        contentType,
        slug: formattedSlug,
        output: layoutOutput,
        pageData,
        pageContains,
        pageHeadings,
        serverlessData
    };
    layoutOutput = await (0, filters_1.applyFilters)('renderItem', layoutOutput, renderItemFilterArgs);
    /* End action */
    const renderItemEndArgs = {
        id,
        contentType,
        slug: formattedSlug,
        output: layoutOutput,
        pageData,
        pageContains,
        pageHeadings,
        serverlessData
    };
    await (0, actions_1.doActions)('renderItemEnd', renderItemEndArgs);
    /* Output */
    return {
        serverlessRender,
        pageData,
        data: {
            slug: formattedSlug,
            output: layoutOutput
        }
    };
};
exports.renderItem = renderItem;
/**
 * Function - loop through all content types to output pages and posts
 *
 * @param {import('./RenderTypes').RenderArgs} args
 * @return {Promise<import('./RenderTypes').RenderReturn[]|import('./RenderTypes').RenderReturn>}
 */
const render = async (args) => {
    /* Fallback output */
    const fallback = [{
            slug: '',
            output: ''
        }];
    /* Props must be object */
    if (!(0, isObject_1.isObjectStrict)(args)) {
        return fallback;
    }
    const { allData, serverlessData, previewData } = args;
    /* Start action */
    await (0, actions_1.doActions)('renderStart', args);
    /* Reset script and style directories */
    config_1.config.scripts.build = {};
    config_1.config.styles.build = {};
    /* Data */
    if (!(0, isObject_1.isObjectStrict)(allData)) {
        return fallback;
    }
    const { navigation, navigationItem, redirect, content } = allData;
    /* Store navigations and items */
    if (navigation !== undefined) {
        config_1.config.navigation = navigation;
    }
    if (navigationItem !== undefined) {
        config_1.config.navigationItem = navigationItem;
    }
    /* Add formation functions */
    const renderFunctions = getRenderFunctions();
    /* Store content data */
    const data = [];
    /* Loop through pages first to set meta info */
    if (serverlessData === undefined) {
        for (let i = 0; i < content.page.length; i += 1) {
            const item = content.page[i];
            const itemObj = (0, isObject_1.isObjectStrict)(item) ? item : {};
            const { parent, archive } = itemObj;
            /* Id */
            const id = (0, isString_1.isStringStrict)(item.id) ? item.id : '';
            /* Archive */
            const archiveType = (0, normalizeContentType_1.normalizeContentType)(archive);
            if ((0, isString_1.isStringStrict)(archiveType) && id !== '') {
                const archiveSlug = item.slug;
                const archiveTitle = item.title;
                const archiveObj = (0, isObject_1.isObjectStrict)(config_1.config.archiveMeta[archiveType]) ? config_1.config.archiveMeta[archiveType] : {};
                config_1.config.archiveMeta[archiveType] = {
                    id,
                    slug: archiveSlug,
                    title: archiveTitle,
                    ...archiveObj
                };
            }
            /* Parent */
            if ((0, isObject_1.isObjectStrict)(parent) && id !== '') {
                const parentSlug = parent.slug;
                const parentTitle = parent.title;
                const parentId = parent.id;
                if ((0, isString_1.isStringStrict)(parentSlug) && (0, isString_1.isStringStrict)(parentTitle) && (0, isString_1.isStringStrict)(parentId)) {
                    config_1.config.parents[id] = {
                        id: parentId,
                        slug: parentSlug,
                        title: parentTitle,
                        contentType: 'page'
                    };
                }
            }
        }
        /* Redirect */
        if ((0, isArray_1.isArrayStrict)(redirect)) {
            redirect.forEach((r) => {
                if (!(0, isObject_1.isObjectStrict)(r)) {
                    return;
                }
                const { redirect: rr = [] } = r;
                if (!(0, isArray_1.isArrayStrict)(rr)) {
                    return;
                }
                config_1.config.redirects.data = config_1.config.redirects.data.concat(rr);
            });
        }
    }
    else {
        const parentsData = await (0, getJson_1.getJsonFile)((0, getPath_1.getPath)('parents', 'store'));
        const archiveMetaData = await (0, getJson_1.getJsonFile)((0, getPath_1.getPath)('archiveMeta', 'store'));
        const formMetaData = await (0, getJson_1.getJsonFile)((0, getPath_1.getPath)('formMeta', 'store'));
        const navigationsData = await (0, getJson_1.getJsonFile)((0, getPath_1.getPath)('navigations', 'store'));
        const navigationItemsData = await (0, getJson_1.getJsonFile)((0, getPath_1.getPath)('navigationItems', 'store'));
        if (parentsData !== undefined) {
            Object.keys(parentsData).forEach((s) => {
                config_1.config.parents[s] = parentsData[s];
            });
        }
        if (archiveMetaData !== undefined) {
            config_1.config.archiveMeta = archiveMetaData;
        }
        if (formMetaData !== undefined) {
            config_1.config.formMeta = formMetaData;
        }
        if (navigationsData !== undefined) {
            config_1.config.navigation = navigationsData;
        }
        if (navigationItemsData !== undefined) {
            config_1.config.navigationItem = navigationItemsData;
        }
    }
    /* Empty serverless reload */
    Object.keys(config_1.config.serverless.routes).forEach((r) => {
        config_1.config.serverless.routes[r] = [];
    });
    /* Loop through all content types */
    const contentTypes = Object.keys(content);
    for (let c = 0; c < contentTypes.length; c += 1) {
        const contentType = contentTypes[c];
        for (let i = 0; i < content[contentType].length; i += 1) {
            const item = await renderItem({
                item: content[contentType][i],
                contentType,
                serverlessData,
                renderFunctions
            });
            const { serverlessRender = false, data: itemData } = item;
            if (itemData !== undefined) {
                data.push(itemData);
                if (serverlessRender && serverlessData === undefined) {
                    config_1.config.serverless.routes.reload.push({
                        path: itemData.slug.replace(/^\/|\/$/gm, '')
                    });
                }
            }
        }
    }
    /* Files data */
    if (serverlessData === undefined && previewData === undefined) {
        config_1.config.store.files.slugs.data = JSON.stringify(_slugs);
        config_1.config.store.files.parents.data = JSON.stringify(config_1.config.parents);
        config_1.config.store.files.archiveMeta.data = JSON.stringify(config_1.config.archiveMeta);
        config_1.config.store.files.formMeta.data = JSON.stringify(config_1.config.formMeta);
        config_1.config.store.files.navigations.data = JSON.stringify(config_1.config.navigation);
        config_1.config.store.files.navigationItems.data = JSON.stringify(config_1.config.navigationItem);
    }
    /* Output */
    const output = serverlessData !== undefined || previewData !== undefined ? data[0] : data;
    await (0, actions_1.doActions)('renderEnd', { ...args, data: output });
    return output;
};
exports.render = render;
