"use strict";
/**
 * Utils - Filters
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.setFilters = exports.resetFilters = exports.applyFilters = exports.removeFilter = exports.addFilter = exports.filters = void 0;
const isArray_1 = require("../isArray/isArray");
const isString_1 = require("../isString/isString");
const isObject_1 = require("../isObject/isObject");
const isFunction_1 = require("../isFunction/isFunction");
/**
 * Store filter callbacks by name
 *
 * @type {import('./filtersTypes').FiltersFunctions}
 */
let filters = {
    columnProps: [],
    containerProps: [],
    fieldProps: [],
    formProps: [],
    richTextProps: [],
    richTextOutput: [],
    richTextContentItem: [],
    richTextContent: [],
    richTextContentOutput: [],
    renderItem: [],
    renderContent: [],
    renderContentStart: [],
    renderContentEnd: [],
    ajaxRes: [],
    cacheData: []
};
exports.filters = filters;
/**
 * Function - add filter to filters object
 *
 * @param {string} name
 * @param {function} filter
 * @return {boolean}
 */
const addFilter = (name, filter) => {
    if (!(0, isString_1.isStringStrict)(name) || !(0, isFunction_1.isFunction)(filter)) {
        return false;
    }
    if (filters[name] === undefined) {
        filters[name] = [];
    }
    filters[name].push(filter);
    return true;
};
exports.addFilter = addFilter;
/**
 * Function - remove filter from filters object
 *
 * @param {string} name
 * @param {function} filter
 * @return {boolean}
 */
const removeFilter = (name, filter) => {
    if (!(0, isString_1.isStringStrict)(name) || !(0, isFunction_1.isFunction)(filter)) {
        return false;
    }
    const callbacks = filters[name];
    if ((0, isArray_1.isArrayStrict)(callbacks)) {
        const index = callbacks.indexOf(filter);
        if (index > -1) {
            filters[name].splice(index, 1);
            return true;
        }
    }
    return false;
};
exports.removeFilter = removeFilter;
/**
 * Function - update value from callback return values
 *
 * @param {string} name
 * @param {*} value
 * @param {*} [args]
 * @return {Promise<*>}
 */
const applyFilters = async (name, value, args) => {
    const callbacks = filters[name];
    if ((0, isArray_1.isArrayStrict)(callbacks)) {
        for (let i = 0; i < callbacks.length; i += 1) {
            const callback = callbacks[i];
            if ((0, isFunction_1.isFunction)(callback)) {
                value = await callback(value, args);
            }
        }
    }
    return value;
};
exports.applyFilters = applyFilters;
/**
 * Function - empty filters object
 *
 * @return {void}
 */
const resetFilters = () => {
    exports.filters = filters = {
        columnProps: [],
        containerProps: [],
        fieldProps: [],
        formProps: [],
        richTextProps: [],
        richTextOutput: [],
        richTextContentItem: [],
        richTextContent: [],
        richTextContentOutput: [],
        renderItem: [],
        renderContent: [],
        renderContentStart: [],
        renderContentEnd: [],
        ajaxRes: [],
        cacheData: []
    };
};
exports.resetFilters = resetFilters;
/**
 * Function - fill filters object
 *
 * @param {import('./filtersTypes').Filters} args
 * @return {boolean}
 */
const setFilters = (args) => {
    if (!(0, isObject_1.isObjectStrict)(args)) {
        return false;
    }
    if (Object.keys(args).length === 0) {
        return false;
    }
    resetFilters();
    Object.keys(args).forEach((a) => {
        const arg = args[a];
        if (arg === undefined) {
            return;
        }
        addFilter(a, arg);
    });
    return true;
};
exports.setFilters = setFilters;
