"use strict";
/**
 * Utils - Get All Contentful Data
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllContentfulData = void 0;
const config_1 = require("../../config/config");
const getContentfulData_1 = require("../getContentfulData/getContentfulData");
const isArray_1 = require("../isArray/isArray");
const isObject_1 = require("../isObject/isObject");
const isString_1 = require("../isString/isString");
const isFunction_1 = require("../isFunction/isFunction");
const getJson_1 = require("../getJson/getJson");
const getPath_1 = require("../getPath/getPath");
/**
 * Function - fetch data from all content types or single entry if serverless
 *
 * @param {import('./getAllContentfulDataTypes').AllContentfulDataArgs} args
 * @return {Promise<import('../../render/RenderTypes').RenderAllData|undefined>}
 */
const getAllContentfulData = async (args = {}) => {
    const { serverlessData, previewData, filterData, filterAllData } = args;
    try {
        /* Store all data */
        let allData = {
            navigation: [],
            navigationItem: [],
            redirect: [],
            content: {
                page: []
            }
        };
        /* Get single entry data if serverless or preview data */
        let isEntry = false;
        if (serverlessData !== undefined || previewData !== undefined) {
            let contentType = '';
            let id = '';
            if (serverlessData !== undefined) {
                const slugsData = await (0, getJson_1.getJsonFile)((0, getPath_1.getPath)('slugs', 'store'));
                const path = serverlessData.path;
                if (slugsData !== undefined) {
                    const item = slugsData[path];
                    if ((0, isObject_1.isObjectStrict)(item)) {
                        id = (0, isString_1.isStringStrict)(item.id) ? item.id : '';
                        contentType = (0, isString_1.isStringStrict)(item.contentType) ? item.contentType : '';
                    }
                }
            }
            if (previewData !== undefined) {
                id = previewData.id;
                contentType = previewData.contentType;
            }
            if (id !== '') {
                const key = `serverless_${id}`;
                const params = {
                    'sys.id': id,
                    include: 10
                };
                const data = await (0, getContentfulData_1.getContentfulData)(key, params);
                if ((0, isArray_1.isArray)(data.items)) {
                    isEntry = true;
                    allData.content[contentType] = data.items;
                }
            }
        }
        /* Get partial data - not serverless */
        if (serverlessData === undefined || !isEntry) {
            const partial = config_1.config.partialTypes;
            for (let i = 0; i < partial.length; i += 1) {
                const contentType = partial[i];
                allData[contentType] = [];
                const key = `all_${contentType}`;
                const params = {
                    content_type: contentType
                };
                let data = await (0, getContentfulData_1.getContentfulData)(key, params);
                if ((0, isFunction_1.isFunction)(filterData)) {
                    data = filterData(data, serverlessData, previewData);
                }
                if ((0, isArray_1.isArray)(data.items)) {
                    allData[contentType] = data.items;
                }
            }
        }
        /* Get whole data (for page generation) - not serverless or preview */
        if ((serverlessData === undefined && previewData === undefined) || !isEntry) {
            const whole = config_1.config.wholeTypes;
            for (let i = 0; i < whole.length; i += 1) {
                const contentType = whole[i];
                allData.content[contentType] = [];
                const key = `all_${contentType}`;
                const params = {
                    content_type: contentType,
                    include: 10
                };
                let data = await (0, getContentfulData_1.getContentfulData)(key, params);
                if ((0, isFunction_1.isFunction)(filterData)) {
                    data = filterData(data, serverlessData, previewData);
                }
                if ((0, isArray_1.isArray)(data.items)) {
                    allData.content[contentType] = data.items;
                }
            }
        }
        /* Filter all data */
        if ((0, isFunction_1.isFunction)(filterAllData)) {
            allData = filterAllData(allData, serverlessData, previewData);
        }
        /* Output */
        return allData;
    }
    catch (error) {
        console.error(config_1.config.console.red, '[SSF] Error getting all Contentful data: ', error);
    }
};
exports.getAllContentfulData = getAllContentfulData;
