"use strict";
/**
 * Utils - Get All Data
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllFileData = void 0;
const promises_1 = require("node:fs/promises");
const node_path_1 = require("node:path");
const config_1 = require("../../config/config");
const getFileData_1 = require("../getFileData/getFileData");
const getJson_1 = require("../getJson/getJson");
const undefineProps_1 = require("../undefineProps/undefineProps");
const resolveInternalLinks_1 = require("../resolveInternalLinks/resolveInternalLinks");
const isObject_1 = require("../isObject/isObject");
const isString_1 = require("../isString/isString");
const isArray_1 = require("../isArray/isArray");
const isFunction_1 = require("../isFunction/isFunction");
/**
 * Function - get data from file system
 *
 * @param {import('./getAllFileDataTypes').AllFileDataArgs} args
 * @return {Promise<import('../../render/RenderTypes').RenderAllData|undefined>}
 */
const getAllFileData = async (args) => {
    args = (0, isObject_1.isObjectStrict)(args) ? args : {};
    const { resolveProps = {
        image: ['image'],
        data: ['items', 'internalLink', 'parent']
    }, excludeProps = {
        data: ['content'],
        archive: {
            posts: ['content'],
            terms: ['content']
        }
    }, filterData, filterAllData } = args;
    try {
        /* Get data */
        let data = await (0, getFileData_1.getFileData)('all_file_data', { all: true });
        /* Store all data */
        let allData = {
            navigation: [],
            navigationItem: [],
            redirect: [],
            content: {
                page: []
            }
        };
        config_1.config.partialTypes.forEach((contentType) => {
            allData[contentType] = [];
        });
        config_1.config.wholeTypes.forEach((contentType) => {
            allData.content[contentType] = [];
        });
        /* Data must be non empty object */
        if (!(0, isObject_1.isObjectStrict)(data) || Object.keys(data).length === 0) {
            throw new Error('No file data');
        }
        /* Image data */
        let imageData = {};
        if ((0, isString_1.isStringStrict)(config_1.config.static.image.dataFile)) {
            const imageDataContents = await (0, promises_1.readFile)((0, node_path_1.resolve)(config_1.config.static.image.dataFile), { encoding: 'utf8' });
            const imageDataJson = (0, getJson_1.getJson)(imageDataContents);
            if (imageDataJson !== undefined) {
                imageData = imageDataJson;
            }
        }
        (0, resolveInternalLinks_1.resolveInternalLinks)(imageData, data, resolveProps.image);
        /* Id */
        Object.keys(data).forEach((d) => {
            const dataItem = data[d];
            if ((0, isObject_1.isObjectStrict)(dataItem)) {
                const { contentType } = dataItem;
                if ((0, isString_1.isStringStrict)(contentType)) {
                    dataItem.id = d;
                }
            }
        });
        /* Internal props */
        resolveProps.data.forEach((d) => {
            (0, resolveInternalLinks_1.resolveInternalLinks)(data, data, [d], (prop, value) => {
                if (resolveProps.data.includes(prop)) {
                    const newValue = (0, undefineProps_1.undefineProps)(value, excludeProps.data);
                    return newValue;
                }
                return value;
            });
        });
        /* Filter data */
        if ((0, isFunction_1.isFunction)(filterData)) {
            data = filterData(data);
        }
        /* Empty archive data */
        const archivePosts = {};
        // const archiveTerms: any = {}
        /* Set content */
        Object.keys(data).forEach((d) => {
            const dataItem = data[d];
            if (!(0, isObject_1.isObjectStrict)(dataItem)) {
                return;
            }
            const { contentType } = dataItem;
            if (!(0, isString_1.isStringStrict)(contentType)) {
                return;
            }
            const partial = allData[contentType];
            if ((0, isArray_1.isArray)(partial)) {
                partial.push(dataItem);
            }
            const whole = allData.content[contentType];
            if ((0, isArray_1.isArray)(whole)) {
                whole.push(dataItem);
            }
            /* Archive */
            if (config_1.config.archiveMeta[contentType] !== undefined) {
                const dataItemCopy = (0, undefineProps_1.undefineProps)(dataItem, excludeProps.archive.posts);
                if (archivePosts[contentType] === undefined) {
                    archivePosts[contentType] = [];
                }
                archivePosts[contentType].push(dataItemCopy);
            }
        });
        /* Term content */
        /*
        Object.keys(config.contentTypes.taxonomy).forEach((tax) => {
          const { contentTypes, props } = config.contentTypes.taxonomy[tax]
    
          if (archiveTerms.terms[tax] === undefined) {
            archiveTerms.terms[tax] = {}
          }
    
          contentTypes.forEach((ct, i) => {
            const contentData = allData.content[ct]
    
            if (isArrayStrict(contentData)) {
              if (archiveTerms.terms[tax][ct] === undefined) {
                archiveTerms.terms[tax][ct] = {}
              }
    
              contentData.forEach((cd) => {
                const prop = props[i]
                const terms = cd[prop] as Generic[]
                const dataCopy = undefineProps(cd, excludeProps.archive.terms)
    
                if (isArrayStrict(terms)) {
                  terms.forEach((term) => {
                    if (!isObjectStrict(term)) {
                      return
                    }
    
                    const termId = term.id
    
                    if (!isStringStrict(termId)) {
                      return
                    }
    
                    if (archiveTerms[tax][ct][termId] === undefined) {
                      archiveTerms[tax][ct][termId] = []
                    }
    
                    const archiveTaxContentTypeTerm = archiveTerms[tax][ct][termId]
    
                    if (isArray(archiveTaxContentTypeTerm)) {
                      archiveTaxContentTypeTerm.push(dataCopy)
                    }
                  })
                }
              })
            }
          })
        })
        */
        /* Filter all data */
        if ((0, isFunction_1.isFunction)(filterAllData)) {
            allData = filterAllData(allData);
        }
        /* Output */
        return allData;
    }
    catch (error) {
        console.error(config_1.config.console.red, '[SSF] Error getting all file data: ', error);
        /* Output */
        return undefined;
    }
};
exports.getAllFileData = getAllFileData;
