"use strict";
/**
 * Utils - Remote Images
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.remoteImages = void 0;
const promises_1 = require("node:fs/promises");
const node_path_1 = require("node:path");
const node_fs_1 = require("node:fs");
const config_1 = require("../../config/config");
const isString_1 = require("../isString/isString");
const isArray_1 = require("../isArray/isArray");
/**
 * Function - promisify write function
 *
 * @private
 * @param {string} path
 * @param {Buffer} buffer
 * @return {Promise<void>}
 */
const _createFile = async (path, buffer) => {
    return await new Promise((resolve, reject) => {
        (0, node_fs_1.createWriteStream)(path).write(buffer, (error) => {
            if (error !== null && error !== undefined) {
                reject(error);
            }
            else {
                resolve();
            }
        });
    });
};
/**
 * Function - download remote images to local images directory
 *
 * @param {import('./remoteImagesTypes').Images[]} images
 * @return {Promise<PromiseSettledResult<void>[]>}
 */
const remoteImages = async (images) => {
    try {
        const inputDir = config_1.config.static.image.inputDir;
        /* Input directory required */
        if (!(0, isString_1.isStringStrict)(inputDir)) {
            throw new Error('No input directory');
        }
        /* Array of image objects required */
        if (!(0, isArray_1.isArrayStrict)(images)) {
            throw new Error('No images array');
        }
        /* Fetch and write images */
        return await Promise.allSettled(images.map(async (image) => {
            const { path, url, ext = 'jpg' } = image;
            if (!(0, isString_1.isStringStrict)(path) || !(0, isString_1.isStringStrict)(url) || !(0, isString_1.isStringStrict)(ext)) {
                return;
            }
            const resp = await fetch(url);
            if (resp.ok) {
                const buffer = await resp.arrayBuffer();
                const fullPath = `${inputDir}/${path}.${ext}`;
                const folders = fullPath.split('/');
                folders.pop();
                await (0, promises_1.mkdir)((0, node_path_1.resolve)(folders.join('/')), { recursive: true });
                return await _createFile((0, node_path_1.resolve)(fullPath), Buffer.from(buffer));
            }
        }));
    }
    catch (error) {
        console.error(config_1.config.console.red, '[SSF] Error downloading remote images: ', error);
    }
    return [];
};
exports.remoteImages = remoteImages;
