/**
 * Utils - Filters
 */
import { isArrayStrict } from '../isArray/isArray';
import { isStringStrict } from '../isString/isString';
import { isObjectStrict } from '../isObject/isObject';
import { isFunction } from '../isFunction/isFunction';
/**
 * Store filter callbacks by name
 *
 * @type {import('./filtersTypes').FiltersFunctions}
 */
let filters = {
    columnProps: [],
    containerProps: [],
    fieldProps: [],
    formProps: [],
    richTextProps: [],
    richTextOutput: [],
    richTextContentItem: [],
    richTextContent: [],
    richTextContentOutput: [],
    renderItem: [],
    renderContent: [],
    renderContentStart: [],
    renderContentEnd: [],
    ajaxRes: [],
    cacheData: []
};
/**
 * Function - add filter to filters object
 *
 * @param {string} name
 * @param {function} filter
 * @return {boolean}
 */
const addFilter = (name, filter) => {
    if (!isStringStrict(name) || !isFunction(filter)) {
        return false;
    }
    if (filters[name] === undefined) {
        filters[name] = [];
    }
    filters[name].push(filter);
    return true;
};
/**
 * Function - remove filter from filters object
 *
 * @param {string} name
 * @param {function} filter
 * @return {boolean}
 */
const removeFilter = (name, filter) => {
    if (!isStringStrict(name) || !isFunction(filter)) {
        return false;
    }
    const callbacks = filters[name];
    if (isArrayStrict(callbacks)) {
        const index = callbacks.indexOf(filter);
        if (index > -1) {
            filters[name].splice(index, 1);
            return true;
        }
    }
    return false;
};
/**
 * Function - update value from callback return values
 *
 * @param {string} name
 * @param {*} value
 * @param {*} [args]
 * @return {Promise<*>}
 */
const applyFilters = async (name, value, args) => {
    const callbacks = filters[name];
    if (isArrayStrict(callbacks)) {
        for (let i = 0; i < callbacks.length; i += 1) {
            const callback = callbacks[i];
            if (isFunction(callback)) {
                value = await callback(value, args);
            }
        }
    }
    return value;
};
/**
 * Function - empty filters object
 *
 * @return {void}
 */
const resetFilters = () => {
    filters = {
        columnProps: [],
        containerProps: [],
        fieldProps: [],
        formProps: [],
        richTextProps: [],
        richTextOutput: [],
        richTextContentItem: [],
        richTextContent: [],
        richTextContentOutput: [],
        renderItem: [],
        renderContent: [],
        renderContentStart: [],
        renderContentEnd: [],
        ajaxRes: [],
        cacheData: []
    };
};
/**
 * Function - fill filters object
 *
 * @param {import('./filtersTypes').Filters} args
 * @return {boolean}
 */
const setFilters = (args) => {
    if (!isObjectStrict(args)) {
        return false;
    }
    if (Object.keys(args).length === 0) {
        return false;
    }
    resetFilters();
    Object.keys(args).forEach((a) => {
        const arg = args[a];
        if (arg === undefined) {
            return;
        }
        addFilter(a, arg);
    });
    return true;
};
/* Exports */
export { filters, addFilter, removeFilter, applyFilters, resetFilters, setFilters };
